<?php
namespace Teknisa\FIN\Modules\ContaReceber\Services\Retorno\Banco;

use Financeiro\Entities\Boletreceber;
use Teknisa\FIN\Modules\ContaReceber\Services\Retorno\LeitorRetorno;
use Teknisa\FIN\Modules\ContaReceber\Services\Retorno\Retorno;

class LeitorRetornoCaixa extends LeitorRetorno{

    const ID_TP_ARQUIVO_HEADER_ARQUIVO = '0';
    const ID_TP_ARQUIVO_HEADER_LOTE = '1';
    const ID_TP_ARQUIVO_DETALHE = '3';
    CONST ID_TP_TRAILER_LOTE = '5';
    CONST ID_TP_TRAILER_ARQUIVO = '9';
    const CODIGO_COMPENSACAO_CAIXA = '104';

    private $arquivo;
    /** @var $retorno Retorno */
    private $retorno;
    private $cdfilial;
    private $nrlanbolrec;

    /**
     * @param $arquivo
     * @param $cdempresa
     * @param $cdbanco
     * @throws \Exception
     */
    public function __construct($arquivo, $cdempresa, $cdbanco, $nrorg){
        try {
            $this->cdbanco = $cdbanco;
            $this->arquivo = $arquivo;
            $this->cdempresa = $cdempresa;
            $this->nrorg = $nrorg;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getEstruturaArquivo(){
        return array(
            self::ID_TP_ARQUIVO_HEADER_ARQUIVO,
            self::ID_TP_ARQUIVO_HEADER_LOTE,
            self::ID_TP_ARQUIVO_DETALHE,
            self::ID_TP_TRAILER_LOTE,
            self::ID_TP_TRAILER_ARQUIVO
        );
    }

    public function leArquivo(){
        try {
            foreach ($this->arquivo as $linha) {
                $cont = $this->contlinha++;
                if ($linha){
                    $idsegmento = substr($linha, 13, 1);
                    if($idsegmento == 'U')
                        $cont = $cont - 1;
                    else
                        $this->retorno = new Retorno();


                    // Busca o Tipo de Registro do Arquivo
                    $idRegistro = substr($linha, 7, 1);
                    if(self::validaArquivo($linha, self::ID_TP_ARQUIVO_HEADER_ARQUIVO, self::getEstruturaArquivo(), $this->contlinha)) {
                        switch ($idRegistro) {

                            case(self::ID_TP_ARQUIVO_HEADER_ARQUIVO):
                                self::leHeaderArquivo($linha);
                                break;

                            case(self::ID_TP_ARQUIVO_HEADER_LOTE):
                                self::leHeaderLote($linha);
                                break;
                            case(self::ID_TP_ARQUIVO_DETALHE):
                                self::leDetalhe($linha);
                                break;
                        }
                    }
                    $this->retornos[$cont] = $this->retorno;
                }
            }
            self::preparaOcorrencias($this->cdbanco, $this->nrorg);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    //########################################################################
    //############# Identificação do Registro Header #########################
    //########################################################################
    private function leHeaderArquivo($header){

        // CAMPO: Código do BCO na compensação
        // POSIÇÃO: 1 a 3
        // PICTURE: 9(3)
        if(substr($header, 0, 3) != self::CODIGO_COMPENSACAO_CAIXA)
            self::gravaErros($this->contlinha, 'Banco Inválido.', true);

        // CAMPO: Identificação do Arquivo de Retorno
        // POSIÇÃO: 143 a 143
        // PICTURE: 9(1)
        if (substr($header, 142, 1) != '2'){
            self::gravaErros($this->contlinha, 'Arquivo inválido.', true);
        }
    }

    //########################################################################
    //############# Identificação do Registro Header de Lote #################
    //########################################################################
    private function leHeaderLote($header){

        // CAMPO: Tipo de Inscrição da empresa
        // POSIÇÃO: 018 a 018
        // PICTURE: 9(1)
        $tpinscricao = trim(substr($header, 17, 1));
        if (empty($tpinscricao)){
            self::gravaErros($this->contlinha, 'Tipo de inscrição da empresa inválido.', true);
        }


        // CAMPO: Inscrição da empresa
        // POSIÇÃO: 019 a 033
        // PICTURE: 9(15)
        $inscricao = trim(substr($header, 18, 15));
        if (empty($inscricao)){
            self::gravaErros($this->contlinha, 'Inscrição da empresa inválida.', true);
        }
    }

    //########################################################################
    //############# Identificação do Registro Detalhe ########################
    //########################################################################
    private function leDetalhe($detalhe){
        $validaOcorre = substr($detalhe, 15, 02);
        if($validaOcorre != '28'){
            $segmento = substr($detalhe, 13, 1);
            if($segmento == 'T'){
                $this->segmentoT($detalhe, $validaOcorre);
            }
            else{
                $this->segmentoU($detalhe, $validaOcorre);
            }
        }
    }

    private function segmentoT($detalhe, $validaOcorre){
        $this->cdfilial = substr($detalhe, 105, 4);
        $this->nrlanbolrec = substr($detalhe, 109, 10);
        $idtitulobanco = trim(substr($detalhe, 46, 11));
        $idcartbol = substr($detalhe, 57, 1);
        $cdempresa = $this->cdempresa;
        $cdNosNum = trim(substr($detalhe,  41, 15));
        $nossonumero = Boletreceber::getRepository();
        /** @var $nossonumero \Financeiro\Repositories\Boletreceber */
        $cdnosnumrec = $nossonumero->getNossoNumero($idtitulobanco, $cdempresa);
        $boleto = false;
        if(!empty($this->cdfilial) && !empty($this->nrlanbolrec))
            /** @var $boleto Boletreceber */
            $boleto = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg($this->nrlanbolrec, $this->cdfilial, $this->nrorg);
        elseif($cdnosnumrec)
            /** @var $boleto Boletreceber */
            $boleto = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg(
                $cdnosnumrec[0]['NRLANBOLREC'], $cdnosnumrec[0]['CDFILIAL'], $this->nrorg
            );
        $this->retorno->setBoleto('BOLETO');
        $this->retorno->setCdlinha(str_pad($this->contlinha, 4, 0, STR_PAD_LEFT));
        $this->retorno->setCdfilial($this->cdfilial);
        $this->retorno->setNrlanbolrec($this->nrlanbolrec);
        $this->retorno->setNossonumero($cdNosNum);
        $this->retorno->setCarteira($idcartbol);
        //$this->retorno->setCdagencia(trim(substr($detalhe, 17, 6)));
        //$this->retorno->setCdconta(trim(substr($detalhe, 23, 13)));
        if($boleto) {
            $this->retorno->setVrfinalbrec($boleto->getVrfinalbrec());
            $this->retorno->setVrbolrec($boleto->getVrbolrec());
            $this->retorno->setIdstatrec($boleto->getIdstatbrec());
            $this->retorno->setCdfilial($boleto->getCdfilial());
            $this->retorno->setNrlanbolrec($boleto->getNrlanbolrec());
            $this->retorno->setCdtpbaixarec($boleto->getCdtpbaixarec());
            $this->retorno->setCdreducont($boleto->getCdreducont());
        }
        if (empty($nossonumero))
            $this->gravaErros($this->contlinha, 'Nosso número inválido.');
        $idocorrencia = trim(substr($detalhe, 15, 2));
        if(empty($idocorrencia))
            $this->gravaErros($this->contlinha, 'Identificação da Ocorrência do banco.');
        $this->retorno->setVrnominal(round(floatval(substr($detalhe, 81, 15)) / 100, 2));
        $this->retorno->setVrdesconto(round(floatval(substr($detalhe, 198, 15)) / 100, 2));
        /*doVrDesconto := 0;
        //Não considera os valores de tarifas como ajuste do boleto
        if cdsRetornoVRDESCONTO.Value > 0 then begin
            doVrDesconto := RoundDec(cdsRetornoVRDESCONTO.Value, 2);
            cdsRetornoVRDESCONTO.Value := 0;
        end;*/
        $iderros = trim(substr($detalhe, 213, 10));
    }

    private function segmentoU($detalhe, $validaOcorre){
        $this->retorno->setVracrescimo(
            round(floatval(substr($detalhe, 17, 15)) / 100, 2) +
            round(floatval(substr($detalhe, 122, 15)) / 100, 2)
        );
        $this->retorno->setVrdesconto(
            round(floatval(substr($detalhe, 32, 15)) / 100, 2) +
            round(floatval(substr($detalhe, 47, 15)) / 100, 2)
        );
        $this->retorno->setVrpago(round(floatval(substr($detalhe, 77, 15)) / 100, 2));
        //Não considera o valor de tarifas sobre o valor total pago
        if($this->retorno->getVrdesconto() > 0)
            $this->retorno->setVrpago($this->retorno->getVrpago() - ($this->retorno->getVrdesconto() * -1));
        if($validaOcorre != '28'){
            $this->retorno->setVrdesconto($this->retorno->getVrdesconto() + substr($detalhe, 107, 15));
        }
        $dtocorrencia = substr($detalhe, 137, 8);
        $dtocorrencia = substr($dtocorrencia, 0, 2) . "/" . substr($dtocorrencia, 2, 2) . "/" . substr($dtocorrencia, -4);
        $this->retorno->setDtocorrencia($dtocorrencia);
        $situacao = self::situacaoCnab($validaOcorre);
        $this->retorno->setIdsitucnabr($situacao);
        if($situacao == '90'){
            $vencimento = substr($detalhe, 145, 8);
            $vencimento = substr($vencimento, 0, 2) . "/" . substr($vencimento, 1, 2) . "/" . substr($vencimento, -4);
            $this->retorno->setDtvencimento($vencimento);
        }
    }

    /**
     * @param $cdocorrencia
     * @return string
     */
    private function situacaoCnab($cdocorrencia){
        if($cdocorrencia == '05' || $cdocorrencia == '09')
            return '00'; //Não Agendado
        else if($cdocorrencia == '02' or $cdocorrencia == '14' or $cdocorrencia == '19' or $cdocorrencia == '23')
            return '20'; //Retorno OK
        else if($cdocorrencia == '03' or $cdocorrencia == '26' or $cdocorrencia == '30' or $cdocorrencia == '45')
            return '01'; //Retorno com Erro
        else if($cdocorrencia == '06' or $cdocorrencia == '17')
            return '90'; //Baixado
        return '10'; //Sem retorno
    }

}